package intensity;
import java.io.Serializable;
import java.util.Comparator;
import java.util.List;

import org.apache.commons.math3.util.Pair;


/**
 * Helper functions related to the Java Collections Framework 
 */
public class CollectionUtil {
	/**
	 * convert list of Doubles to array of primitive doubles
	 * @param values
	 * @return
	 */
	public static final double[] toArray(List<Double> values) {
		double[] array = new double[values.size()];
		int i = 0;
		for (Double value: values)
			array[i++] = value;
		return array;
	}
	
  /**
	 * comparator that compares pairs by comparing the value objects
	 *
	 * @param <T1> type 1
	 * @param <T2> type 2 
	 */
	public final static class PairValueComparator<T1,T2 extends Comparable<T2>> implements Comparator<Pair<T1,T2>>, Serializable {
		private static final long serialVersionUID = 1L;
	
		/* (non-Javadoc)
		 * @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
		 */
		@Override
		public final int compare(Pair<T1, T2> o1, Pair<T1,T2> o2) {
			T2 second1 = o1.getValue();
			T2 second2 = o2.getValue();
			if (second1 == null) {
				if (second2 == null)
					return 0;
				else
					return 1;
			}
			return second1.compareTo(second2);
		}
	}

}
