package intensity;


import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * single iterator class: an instance of an Iterator that only returns a single
 * item
 * 
 * @param <T> type of objet to return
 */
public class IteratorSingle<T> implements Iterator<T> {
	T object;
	boolean returned = false;
	
	/**
	 * @param object
	 */
	public IteratorSingle(T object) {
		super();
		this.object = object;
	}

	/* (non-Javadoc)
	 * @see java.util.Iterator#hasNext()
	 */
	@Override
	public final boolean hasNext() {
		return !returned;
	}

	/* (non-Javadoc)
	 * @see java.util.Iterator#next()
	 */
	@Override
	public final T next() throws NoSuchElementException {
		if (!returned) {
			returned = true;
			T objectToReturn = object;
			object = null;
			return objectToReturn;
		}
		throw new NoSuchElementException();
	}

	/** (non-Javadoc)
	 * @see java.util.Iterator#remove()
	 */
	@Override
	public final void remove() {
		if (!returned || object == null)
			throw new IllegalStateException();
		object = null;
	}
}