package intensity;

import gnu.trove.TObjectDoubleHashMap;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;

public class LexicalStatistics {
    private final static String weakToStrongMarker = "totalCnt_weakStrong";
    private final static String strongToWeakMarker = "totalCnt_strongWeak";
    private final static String keyValSep = " --> ";
    private final String entrySep = "x-----x";

    public double weakStrongPatternScore = Double.NaN;
    public double strongWeakPatternScore = Double.NaN;
    
    public TObjectDoubleHashMap<String> wordFreqs = new TObjectDoubleHashMap<String>(); 
    
    public LexicalStatistics(File wordFreqFile, File patternFreqFile) throws NumberFormatException, IOException {
        super();
        
        // read pattern frequencies
        BufferedReader fi = new BufferedReader(new InputStreamReader(new FileInputStream(patternFreqFile), "UTF-8"));
        String line = null;
        while ((line = fi.readLine()) != null) {
            if (line.startsWith(weakToStrongMarker)) {
                int i = line.indexOf(keyValSep);
                weakStrongPatternScore = Double.parseDouble(line.substring(i + keyValSep.length()));
            } else if (line.startsWith(strongToWeakMarker)) {
                int i = line.indexOf(keyValSep);
                strongWeakPatternScore = Double.parseDouble(line.substring(i + keyValSep.length()));
                if (!Double.isNaN(weakStrongPatternScore))
                    break;
            }
        }
        fi.close();
        Logger.out.println("WS Normalization Value: " + weakStrongPatternScore);
        Logger.out.println("SW Normalization Value: " + strongWeakPatternScore);
        
        // read word frequencies
        fi = new BufferedReader(new InputStreamReader(new FileInputStream(wordFreqFile), "UTF-8"));
        while ((line = fi.readLine()) != null) {
            int i = line.indexOf(keyValSep);
            wordFreqs.put(line.substring(0, i).trim(), Double.parseDouble(line.substring(i + keyValSep.length())));
        }
        fi.close();
    }
    
    public Map<String, Double> computeWordPairWeights(File patternOccFile)
            throws UnsupportedEncodingException, FileNotFoundException,
            IOException 
    {
        BufferedReader fi;
        String line;

        // read pattern occurrence frequencies
        HashMap<String,Double> wordPairFreqs = new HashMap<String,Double>();
        fi = new BufferedReader(new InputStreamReader(new FileInputStream(patternOccFile), "UTF-8"));
        String curPair = null;
        while ((line = fi.readLine()) != null) {
            //Logger.out.println(line);
            if (line.equals(entrySep)) {
                String nextLine = fi.readLine();
                int i = nextLine.indexOf(", ");
                curPair = nextLine.substring(1, i).trim() + '\t' + nextLine.substring(i + 2, nextLine.length() - 1).trim();
                wordPairFreqs.put(curPair, 0.0);
            } else if (line.startsWith(weakToStrongMarker)) {
                int i = line.indexOf(keyValSep);
                double score = Double.parseDouble(line.substring(i + keyValSep.length()));
                score /= weakStrongPatternScore;
                wordPairFreqs.put(curPair, wordPairFreqs.get(curPair) + score);
            } else if (line.startsWith(strongToWeakMarker)) {
                int i = line.indexOf(keyValSep);
                double score = Double.parseDouble(line.substring(i + keyValSep.length()));
                score /= strongWeakPatternScore;
                wordPairFreqs.put(curPair, wordPairFreqs.get(curPair) - score);
            }
        }
        fi.close();
        
        Logger.out.println("\nNet Frequencies");
        Map<String,Double> wordPairWeights = new HashMap<String,Double>();
        for (Map.Entry<String, Double> entry: wordPairFreqs.entrySet()) {
            String pairStr = entry.getKey();
            String[] pair = pairStr.split("\t");
            pair[0] = pair[0].trim();
            pair[1] = pair[1].trim();
            
            if (pair[0].compareTo(pair[1]) < 0) {
                double origScore12 = entry.getValue();
                double origScore21 = wordPairFreqs.get(pair[1] + '\t' + pair[0]);
                double score = (origScore12 - origScore21);
                double normalization = wordFreqs.get(pair[0]) * wordFreqs.get(pair[1]);
                if (normalization == 0.0) {
                    if (WordCluster.keepWord(pair[0], wordFreqs) && WordCluster.keepWord(pair[1], wordFreqs))
                        System.err.println("Warning: Normalization 0 for " + pair[0] + " / " + pair[1]);
                    score = 0.0;
                    normalization = 1.0;
                }
                if (score != 0.0)
                    score /= normalization;
                wordPairWeights.put(pairStr, score);
            }
        }
        
        Logger.out.println("\nWeights");
        for (Map.Entry<String, Double> entry: wordPairWeights.entrySet()) {
            String[] pair = entry.getKey().split("\t");
            if (entry.getValue() != 0)
                Logger.out.println(pair[0] + " / " + pair[1] + ": " + entry.getValue());
        }
        return wordPairWeights;
    }

    public TObjectDoubleHashMap<String> getWordFreqs() {
        return wordFreqs;
    }
}
