package intensity;
import java.util.Iterator;

/**
 * Logging class (minimal version)
 */
public class Logger {
	/**
	 * Main logger for logging current operation 
	 */
	public static Logger out = new Logger();
	
	private String indentation = "";
	
	private boolean newLine = true;

	/**
	 * begin process (show message and increase indentation)
	 * @param s message
	 */
	public final void begin(String s) {
		println(s);
		indentation += "  ";
	}
	
	/**
	 * end process
	 */
	public final void end() {
		int len = indentation.length();
		if (len > 0) {
			if (len == 2)
				indentation = "";
			else
				indentation = indentation.substring(0, len - 2);
		}
	}
	
	/**
	 * end process with log message
	 * @param string message
	 */
	public final void end(String string) {
		println(string);
		end();
	}
	
	/**
	 * write to log and perhaps print to screen
	 * @param s string to write
	 */
	public final void print(String s) {
		if (s == null)
			return;
		if (s.length() == 0)
			return;
		if (indentation.length() > 0)
			s = s.replace("\n", "\n" + indentation);
		if (newLine && indentation.length() > 0) {
			// more efficient to repeat the code
		    System.out.print(indentation + s);
		} else {
			System.out.print(s);
		}
		newLine = s.endsWith("\n");
	}
	
	/**
	 * write to log and perhaps print to screen
	 * @param s string to write
	 */
	public final void println(String s) {
		if (s != null && indentation.length() > 0)
			s = s.replace("\n", "\n" + indentation);
		if (newLine && indentation.length() > 0) {
			System.out.println(indentation + s);
		} else {
			System.out.println(s);
		}
		newLine = true;
	}
	
	/**
	 * write number to log/screen (with newline) 
	 * @param i
	 */
	public final void println(int i) {
		println(Integer.toString(i));
	}
	
	/**
     * write number to log/screen (with newline) 
     * @param i
     */
    public final void println(double i) {
        println(Double.toString(i));
    }
	
	/**
	 * write iterable items to log/screen (with newline) 
	 * @param iterable object that is iterable
	 */
	public final void println(Iterable<? extends Object> iterable) {
		Iterator<? extends Object> it = iterable.iterator();
		print("[");
		boolean first = true;
		while (it.hasNext()) {
			if (first)
				first = false;
			else
				print(", ");
			print(it.next().toString());
		}
		println("]");
	}


	/**
	 * write newline to log (and perhaps print to screen)
	 */
	public final void println() {
		System.out.println();
		newLine = true;
	}
	
	/**
	 * log a warning message
	 * @param s string to write
	 */
	public final void printWarning(String s) {
		printError(s);
	}
	
	/**
	 * log a warning message
	 * @param s string to write
	 */
	public final void printlnWarning(String s) {
		printError(s + "\n");
	}
	
	/**
	 * log an error message
	 * @param s string to write
	 */
	public final void printError(String s) {
		if (s != null && indentation.length() > 0)
			s = s.replace("\n", "\n" + indentation);
        System.err.print(indentation + s);
	}
	
	/**
	 * log an error message
	 * @param s string to write
	 */
	public final void printlnError(String s) {
		printError(s + "\n");
	}
	
	/**
	 * output UI string (for progress reports, etc.)
	 * @param string
	 */
	public void printUI(String string) {
		if (newLine && indentation.length() > 0)
			System.out.print(indentation);
		System.out.print(string);
		newLine = string.endsWith("\n");
	}
}
