package intensity;

import gnu.trove.THashMap;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

/**
 * Map supporting multiple values per key.
 * 
 * @param <KeyType> key type
 * @param <ValueType> value type
 */
public class MultiValueMap<KeyType,ValueType> extends THashMap<KeyType,List<ValueType>> {
	/**
	 * serial version ID
	 */
	private static final long serialVersionUID = 3175371121001348091L;
	
	private int expectedEntries = 1;

	/**
	 * empty constructor
	 */
	public MultiValueMap() {
		super();
	}
	
	/**
	 * constructor
	 * @param expectedNumberOfKeys  initial expected number of keys
	 * @param expectedEntriesPerKey  number of items expected for a single key 
	 */
	public MultiValueMap(int expectedNumberOfKeys, int expectedEntriesPerKey) {
		super((expectedNumberOfKeys == 0) ? 20 : expectedNumberOfKeys);
		this.expectedEntries = expectedEntriesPerKey;
	}
	
	/**
	 * put single item into map
	 * @param key
	 * @param value
	 * @return list of values after adding
	 */
	public final List<ValueType> add(KeyType key, ValueType value) {
		List<ValueType> list = get(key);
		if (list != null) {
			list.add(value);
			//put(key, list);
		} else {
			if (expectedEntries > 2)
				list = new ArrayList<ValueType>(expectedEntries);
			else
				list = new LinkedList<ValueType>();
			list.add(value);
			put(key, list);
		}
		return list;
	}
}
