package intensity;

import gnu.trove.TObjectDoubleHashMap;

import java.io.IOException;
import java.text.DecimalFormat;
import java.util.List;
import java.util.Map;

import org.apache.xmlrpc.XmlRpcException;

public abstract class Ranker {

    public Ranker() {
        super();
    }

    /**
     * get ranking
     * @param cluster word cluster
     * @param wordPairWeights word pair relative intensity weights
     * @param wordFreqs word frequencies 
     * @param synonymSets sets of words considered synonymous
     * @throws IOException
     * @throws InterruptedException
     * @throws XmlRpcException
     * @throws Exception
     */
    abstract public Ranking rank(WordCluster cluster, Map<String, Double> wordPairWeights, TObjectDoubleHashMap<String> wordFreqs, List<List<String>> synonymSets) throws Exception;

    protected static boolean isSynonym(List<List<String>> synonymSets,
            String adj1, String adj2)
    {
        if (synonymSets != null)
            for (List<String> synset: synonymSets)
                if (synset.contains(adj1) && synset.contains(adj2))
                    return true;
        return false;
    }
    
    /**
     * format a number
     * @param number number
     * @return number as a string using 4 non-integer decimal places
     */
    public static String formatNumberDefault(double number) {
        return defaultNumberFormatter.format(number);
    }

    private static DecimalFormat defaultNumberFormatter = new DecimalFormat("#0.0000");

}