package intensity;

import gnu.trove.TObjectDoubleHashMap;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Random;

public class RankerDivideAndConquer extends Ranker {
    @Override
    public Ranking rank(WordCluster cluster, Map<String, Double> wordPairWeights, TObjectDoubleHashMap<String> wordFreqs, List<List<String>> synonymSets)
            throws Exception 
    {
        Ranking ranking = new Ranking();
        divideAndConquer(cluster, wordPairWeights, ranking, 0.0, 1.0);
        return ranking;
    }
    
    private void divideAndConquer(WordCluster cluster, Map<String, Double> wordPairWeights, Ranking ranking, double ofs, double range) {
        Random rng = new Random();
        if (cluster.size() == 0) {
            // do nothing
        } else if (cluster.size() == 1) {
            ranking.add(Collections.singletonList(cluster.words.get(0)), ofs + range/2.0);
        } else {
            String pivot = cluster.words.get(rng.nextInt(cluster.words.size()));
            WordCluster clusterBelow = new WordCluster(new LinkedList<String>(), cluster.getOrigSynsets());
            WordCluster clusterEqual = new WordCluster(new LinkedList<String>(), cluster.getOrigSynsets());
            WordCluster clusterAbove = new WordCluster(new LinkedList<String>(), cluster.getOrigSynsets());
            RankerDivideAndConquer.partition(cluster, wordPairWeights, Collections.singletonList(pivot), clusterBelow, clusterEqual, clusterAbove, false);
            // add words to ranking in the right order
            divideAndConquer(clusterBelow, wordPairWeights, ranking, ofs, range/2.0);
            ranking.add(clusterEqual.getWords(), ofs + range/2.0);
            divideAndConquer(clusterAbove, wordPairWeights, ranking, ofs + range/2.0, range/2.0);
        }
    }
    
    /**
     * @param pivotsAsEqual if true, then all the pivots are assigned to the equal cluster (clusterEqual) 
     */
    protected static void partition(WordCluster cluster,
            Map<String, Double> wordPairWeights, List<String> pivots,
            WordCluster clusterBelow, WordCluster clusterEqual,
            WordCluster clusterAbove, boolean pivotsAsEqual) {
        for (String word: cluster.words) {
            double adjPairWeight = 0.0;
            if (pivots != null) {
                for (String pivot: pivots)
                    if (word.equals(pivot)) {
                        if (pivotsAsEqual) {
                            adjPairWeight = 0.0;
                            break;
                        }
                    } else {
                        if (word.compareTo(pivot) < 0) {
                            String adjPair = word + '\t' + pivot;
                            if (wordPairWeights.containsKey(adjPair))
                                adjPairWeight += Math.signum(wordPairWeights.get(adjPair));
                        } else { // if (word.compareTo(pivot) > 0) 
                            String adjPair = pivot + '\t' + word;
                            if (wordPairWeights.containsKey(adjPair))
                                adjPairWeight += Math.signum(-wordPairWeights.get(adjPair));
                        }
                    }
            }
            if (adjPairWeight == 0.0)
                clusterEqual.words.add(word); // in particular, this should happen for the pivot if there is only one pivot
            else if (adjPairWeight > 0.0)
                clusterBelow.words.add(word);
            else 
                clusterAbove.words.add(word);
        }
    }

}
