package intensity;


import java.util.AbstractSet;
import java.util.Collection;
import java.util.Iterator;

/**
 * Mutable Singleton (1-tuple)
 * 
 * For non-mutable singletons use Collections.singleton().
 * 
 * @param <Type> type of element
 */
public class Singleton<Type> extends AbstractSet<Type> {
	private Type element = null;
	
	/**
	 * empty Constructor
	 */
	public Singleton() {
		// do nothing
	}
	
	/**
	 * Constructor
	 * @param element object to store
	 */
	public Singleton(Type element) {
		this.element = element;
	}

	/**
	 * @return The item contained in the singleton set.
	 */
	public final Type getObject() {
		return element;
	}

	/**
	 * @param object The object to place in the set (replacing any previous values).
	 */
	public final void setObject(Type object) {
		this.element = object;
		//hashcode = -1;
	}
	
	// note: AbstractSet.equals() and hashcode() are used
	
	/**
	 * @return string representation
	 */
	@Override
	public final String toString() {
		StringBuilder s = new StringBuilder();
		s.append("Singleton (");
		if (element == null)
			s.append("null");
		else
			s.append(element.toString());
		s.append(")");
		return s.toString();
	}

	/* (non-Javadoc)
	 * @see java.util.AbstractCollection#iterator()
	 */
	@Override
	public final Iterator<Type> iterator() {
		return new IteratorSingle<Type>(element);
	}

	/* (non-Javadoc)
	 * @see java.util.AbstractCollection#size()
	 */
	@Override
	public final int size() {
		return 1;
	}

	/* (non-Javadoc)
	 * @see java.util.AbstractCollection#addAll(java.util.Collection)
	 */
	@Override
	public final boolean addAll(Collection<? extends Type> c) {
		throw new UnsupportedOperationException();
	}

	/* (non-Javadoc)
	 * @see java.util.AbstractCollection#clear()
	 */
	@Override
	public final void clear() {
		throw new UnsupportedOperationException();
	}
	
	/** (non-Javadoc)
	 * @see java.util.AbstractCollection#remove(java.lang.Object)
	 */
	@Override
	public final boolean remove(Object o) {
		if (!contains(o))
			return false;
		throw new UnsupportedOperationException();
	}

	/* (non-Javadoc)
	 * @see java.util.AbstractCollection#contains(java.lang.Object)
	 */
	@Override
	public final boolean contains(Object o) {
		if (element == null)
			return o == null;
		else
			return element.equals(o);
	}
}
