package intensity;

import gnu.trove.TObjectDoubleHashMap;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class RankerSheinmanTokunaga extends Ranker {
    
    @Override
    public Ranking rank(WordCluster cluster, Map<String, Double> wordPairWeights, TObjectDoubleHashMap<String> wordFreqs, List<List<String>> synonymSets)
            throws Exception 
    {
        Ranking ranking = new Ranking();
        divide(cluster, wordPairWeights, wordFreqs, ranking, 0.0, 1.0, 1);
        return ranking;
    }
    
    private void divide(WordCluster cluster, Map<String, Double> wordPairWeights, TObjectDoubleHashMap<String> wordFreqs, Ranking ranking, double ofs, double range, int sheinmanPhase) {
        if (cluster.size() == 0) {
            // do nothing
        } else if (cluster.size() == 1 || sheinmanPhase > 2) {
            // deal with singletons as well as leftover groups after the 2nd phase.
            ranking.add(cluster.words, ofs + range/2.0);
        } else {
            List<String> pivots;
            if (sheinmanPhase == 1) {
                // in the first phase, words are compared with the headwords
                pivots = intersection(cluster.getOrigSynsets().get(0), cluster.words);
                if (pivots.size() == 0) {
                    // if we do not have a headword from WordNet, we use the most frequent
                    // word, as proposed by Sheinman
                    String pivot = null;
                    double pivotFreq = -1;
                    for(String word: cluster) {
                        double wordFreq = wordFreqs.get(word);
                        if (wordFreq > pivotFreq) {
                            pivot = word;
                            pivotFreq = wordFreq;
                        }
                    }
                    pivots = Collections.singletonList(pivot);
                }
            } else { // if (sheinManDepth == 2)
                // in the 2nd phase, we instead do a pairwise comparisons of all words
                pivots = cluster.words;
            }
            WordCluster clusterBelow = new WordCluster(new LinkedList<String>(), cluster.getOrigSynsets());
            WordCluster clusterEqual = new WordCluster(new LinkedList<String>(), cluster.getOrigSynsets());
            WordCluster clusterAbove = new WordCluster(new LinkedList<String>(), cluster.getOrigSynsets());
            RankerDivideAndConquer.partition(cluster, wordPairWeights, pivots, clusterBelow, clusterEqual, clusterAbove, sheinmanPhase == 1);
                // note that in phase 1, all words in the pivot are kept as neutral, because Sheinman & Tokunaga
                // exploit the WordNet synonymy information and only place similar ones in the intense or mild sets. 
            // next, add words to ranking in the right order
            divide(clusterBelow, wordPairWeights, wordFreqs, ranking, ofs, range/2.0, sheinmanPhase + 1);
            ranking.add(clusterEqual.getWords(), ofs + range/2.0);
            divide(clusterAbove, wordPairWeights, wordFreqs, ranking, ofs + range/2.0, range/2.0, sheinmanPhase + 1);
        }
    }
    
    private List<String> intersection(List<String> list1, List<String> list2) {
        List<String> result = new LinkedList<String>();
        for(String s: list1) {
            if(list2.contains(s))
                result.add(s);
        }
        return result;
    }

}
